/********************************************************************/
/*																	*/
/*  CookieViewDlg.cpp												*/
/*																	*/
/*	This class is a part of Cookie Viewer							*/
/*																	*/
/*  Programmed by Pablo van der Meer								*/
/*  Copyright Pablo Software Solutions 2003							*/
/*	http://www.pablovandermeer.nl									*/
/*																	*/
/*  Last updated: 2 January 2003									*/
/*																	*/
/********************************************************************/


#include "stdafx.h"
#include "CookieView.h"
#include "CookieViewDlg.h"
#include "SettingsDlg.h"
#include "AboutDlg.h"
#include "FindDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static UINT auIDStatusBar[] = 
{
   ID_SEPARATOR,
   ID_INDICATOR_TIME
};


BEGIN_DLGRESIZE_MAP(CCookieViewDlg)
	DLGRESIZE_CONTROL(IDC_COOKIES, DLSZ_SIZE_Y)
	DLGRESIZE_CONTROL(IDC_DATA, DLSZ_SIZE_Y)
	BEGIN_DLGRESIZE_GROUP()
		DLGRESIZE_CONTROL(IDC_COOKIES, DLSZ_SIZE_X)
		DLGRESIZE_CONTROL(IDC_DATA, DLSZ_SIZE_X)
	DLGRESIZE_CONTROL(IDC_STATIC0, DLSZ_MOVE_X)
	DLGRESIZE_CONTROL(IDC_STATIC1, DLSZ_MOVE_X)
	DLGRESIZE_CONTROL(IDC_STATIC2, DLSZ_MOVE_X)
	DLGRESIZE_CONTROL(IDC_STATIC3, DLSZ_MOVE_X)
	DLGRESIZE_CONTROL(IDC_STATIC4, DLSZ_MOVE_X)
	DLGRESIZE_CONTROL(IDC_DOMAIN, DLSZ_MOVE_X | DLSZ_SIZE_X)
	DLGRESIZE_CONTROL(IDC_CREATED, DLSZ_MOVE_X | DLSZ_SIZE_X)
	DLGRESIZE_CONTROL(IDC_EXPIRES, DLSZ_MOVE_X | DLSZ_SIZE_X)
	DLGRESIZE_CONTROL(IDC_LIFETIME, DLSZ_MOVE_X | DLSZ_SIZE_X)
	END_DLGRESIZE_GROUP()

	DLGRESIZE_CONTROL(IDC_STATIC0, DLSZ_MOVE_Y)
	DLGRESIZE_CONTROL(IDC_STATIC1, DLSZ_MOVE_Y)
	DLGRESIZE_CONTROL(IDC_STATIC2, DLSZ_MOVE_Y)
	DLGRESIZE_CONTROL(IDC_STATIC3, DLSZ_MOVE_Y)
	DLGRESIZE_CONTROL(IDC_STATIC4, DLSZ_MOVE_Y)
	DLGRESIZE_CONTROL(IDC_CREATED, DLSZ_MOVE_Y)
	DLGRESIZE_CONTROL(IDC_EXPIRES, DLSZ_MOVE_Y)
	DLGRESIZE_CONTROL(IDC_LIFETIME, DLSZ_MOVE_Y)
	DLGRESIZE_CONTROL(IDC_DOMAIN, DLSZ_MOVE_Y)
	
	DLGRESIZE_CONTROL(AFX_IDW_TOOLBAR, DLSZ_SIZE_X)
END_DLGRESIZE_MAP()


CCookieViewDlg::CCookieViewDlg(CWnd* pParent /*=NULL*/)
	: CDialogResize(CCookieViewDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCookieViewDlg)
	m_strDomain = _T("");
	m_strCreated = _T("");
	m_strExpires = _T("");
	m_strLifeTime = _T("");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_nPreviousSelected = -1;
}

void CCookieViewDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogResize::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCookieViewDlg)
	DDX_Control(pDX, IDC_DATA, m_DataList);
	DDX_Control(pDX, IDC_COOKIES, m_CookiesList);
	DDX_Text(pDX, IDC_DOMAIN, m_strDomain);
	DDX_Text(pDX, IDC_CREATED, m_strCreated);
	DDX_Text(pDX, IDC_EXPIRES, m_strExpires);
	DDX_Text(pDX, IDC_LIFETIME, m_strLifeTime);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CCookieViewDlg, CDialogResize)
	//{{AFX_MSG_MAP(CCookieViewDlg)
	ON_COMMAND(ID_FILE_SETTINGS, OnFileSettings)
	ON_NOTIFY(LVN_DELETEITEM, IDC_DATA, OnDeleteitemData)
	ON_COMMAND(ID_FILE_EXIT, OnFileExit)
	ON_BN_CLICKED(ID_EDIT_DELETE, OnEditDelete)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_COOKIES, OnItemchangedCookies)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_DATA, OnItemchangedData)
	ON_COMMAND(ID_EDIT_SELECT_ALL, OnEditSelectAll)
	ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
	ON_COMMAND(ID_VIEW_REFRESH, OnViewRefresh)
	ON_COMMAND(ID_EDIT_FIND, OnEditFind)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/********************************************************************/
/*																	*/
/* Function name : OnInitDialog										*/
/* Description   : Called by the framework in response to the		*/
/*				   WM_INITDIALOG message.							*/
/*																	*/
/********************************************************************/
BOOL CCookieViewDlg::OnInitDialog()
{
	CDialogResize::OnInitDialog();

	// Set big icon
	SetIcon(m_hIcon, TRUE);			
	// Set small icon
	SetIcon(m_hIcon, FALSE);		

	// Add toolbar/statusbar to the dialog
    if (!CreateBars())
    {
		// fail to create the toolbar
        return FALSE;      
	}

	// TODO: Add extra initialization here
	InitResizing();

	m_CookiesList.InsertColumn(0, "Filename");

	DWORD dwStyle = m_CookiesList.GetExtendedStyle();
	dwStyle |= LVS_EX_FULLROWSELECT;
    m_CookiesList.SetExtendedStyle(dwStyle);

	ShowCookies();

	m_DataList.InsertColumn(0, "Cookie Name");
	m_DataList.InsertColumn(1, "Cookie Value");
	m_DataList.InsertColumn(2, "Secure");

	m_DataList.GetExtendedStyle();
	dwStyle |= LVS_EX_FULLROWSELECT;
    m_DataList.SetExtendedStyle(dwStyle);

	AutoSizeColumns(&m_DataList);

	return TRUE;  // return TRUE  unless you set the focus to a control
}


/********************************************************************/
/*																	*/
/* Function name : OnFileSettings									*/
/* Description   : Show settings dialog								*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::OnFileSettings() 
{
	CCookieViewApp *pApp = (CCookieViewApp *)AfxGetApp();

	CSettingsDlg dlg;

	dlg.m_strCookieFolder = pApp->m_strCookieFolder;
	dlg.m_bConfirmation = pApp->m_bConfirmation;

	if (dlg.DoModal() == IDOK)
	{
		pApp->m_strCookieFolder = dlg.m_strCookieFolder;
		pApp->WriteProfileString("Settings", "CookieFolder", dlg.m_strCookieFolder);

		pApp->m_bConfirmation = dlg.m_bConfirmation;
		pApp->WriteProfileInt("Settings", "Confirmation", dlg.m_bConfirmation);

		ShowCookies();
	}
}


/********************************************************************/
/*																	*/
/* Function name : OnFileExit										*/
/* Description   : Close application								*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::OnFileExit() 
{
	OnOK();	
}


/********************************************************************/
/*																	*/
/* Function name : ShowCookies										*/
/* Description   : Show all cookies in specified folder				*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::ShowCookies()
{
	// reset list
	m_CookiesList.DeleteAllItems();

	CCookieViewApp *pApp = (CCookieViewApp *)AfxGetApp();

	CFileFind find;
	
	// list all .txt files
	BOOL bFound = find.FindFile(pApp->m_strCookieFolder + "\\*.txt");
	while (bFound)
	{
		bFound = find.FindNextFile();
		
		// skip "." and ".." 
		if (find.IsDots())
			continue;

		if (!find.IsDirectory())
		{
			m_CookiesList.InsertItem(m_CookiesList.GetItemCount(), find.GetFileName());
		}
	}
	find.Close();

	AutoSizeColumns(&m_CookiesList);

	m_CookiesList.SetItemState(0, LVIS_SELECTED | LVIS_FOCUSED , LVIS_SELECTED | LVIS_FOCUSED); 
}


/********************************************************************/
/*																	*/
/* Function name : OnItemchangedCookies								*/
/* Description   : Called when selected cookie has changed			*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::OnItemchangedCookies(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	// get selected user
	int nIndex = m_CookiesList.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED | LVIS_FOCUSED); 
	if (nIndex != -1 && m_nPreviousSelected != nIndex)
	{
		m_nPreviousSelected = nIndex;
		CCookieViewApp *pApp = (CCookieViewApp *)AfxGetApp();

		CString strFilename;
		strFilename.Format("%s\\%s", pApp->m_strCookieFolder, m_CookiesList.GetItemText(nIndex, 0));

		AnalyseCookie(strFilename);
	}		

	*pResult = 0;
}


/********************************************************************/
/*																	*/
/* Function name : OnClickData										*/
/* Description   : Called when user clicked data list				*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::OnItemchangedData(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	m_strDomain = "";
	m_strCreated = "";
	m_strExpires = "";
	m_strLifeTime = "";
	UpdateData(FALSE);

    // get index of selected item
	int nIndex = m_DataList.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED); 
    
    if(nIndex != -1)
	{
		ShowCookieDetails((LPCTSTR)m_DataList.GetItemData(nIndex));	
	}
	
	*pResult = 0;
}


/********************************************************************/
/*																	*/
/* Function name : AnalyseCookie									*/
/* Description   : Load cookie and show it's data					*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::AnalyseCookie(LPCTSTR lpszFilename)
{
	m_DataList.DeleteAllItems();

	CFile file;
	
	CString strCookieData;
	if (file.Open(lpszFilename, CFile::modeRead))
	{
		DWORD dwLength = file.GetLength();

		file.Read(strCookieData.GetBuffer(dwLength), dwLength);
		strCookieData.ReleaseBuffer();
		file.Close();
	}

	CString strCookie;
	int nCount = 0;
	
	while(AfxExtractSubString(strCookie, strCookieData, nCount++, '*'))
	{
		if (!strCookie.IsEmpty())
		{
			// remove first linefeed
			strCookie.TrimLeft('\n');

			CStringArray strArray;
			int nCount = Split(strCookie, '\n', strArray);
			if (nCount < 8)
				continue;
			
			int nIndex = m_DataList.InsertItem(m_DataList.GetItemCount(), strArray[0]);
			m_DataList.SetItemText(nIndex, 1, strArray[1]);
			m_DataList.SetItemText(nIndex, 2, strArray[3] ? "True" : "False");
			// save cookie data
			LPTSTR lpszData = new char[strCookie.GetLength()+1];
			lstrcpy(lpszData, strCookie);
			m_DataList.SetItemData(nIndex, (DWORD)lpszData);
		}
	}
	m_DataList.SetItemState(0, LVIS_SELECTED | LVIS_FOCUSED , LVIS_SELECTED | LVIS_FOCUSED); 
}


/********************************************************************/
/*																	*/
/* Function name : ShowCookieDetails								*/
/* Description   : Show details of selected cookie					*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::ShowCookieDetails(LPCTSTR lpszCookie)
{
	CString strCookie = lpszCookie;

	CStringArray strArray;
	int nCount = Split(lpszCookie, '\n', strArray);
	if (nCount < 8)
	{
		return;
	}

	FILETIME ftCreated, ftExpires;

	ftExpires.dwLowDateTime = atol(strArray[4]);
	ftExpires.dwHighDateTime = atol(strArray[5]);
	ftCreated.dwLowDateTime = atol(strArray[6]);
	ftCreated.dwHighDateTime = atol(strArray[7]);
	
	COleDateTime tmCreated(ftCreated);
	m_strCreated = tmCreated.Format();
	COleDateTime tmExpires(ftExpires);
	m_strExpires = tmExpires.Format();

	COleDateTimeSpan lifeTime = tmExpires - tmCreated;
	
	if (lifeTime.GetStatus() == COleDateTimeSpan::valid)
	{
		m_strLifeTime.Format("%d days, %d hours, %d minutes, %d seconds", lifeTime.GetDays(),
 																	  lifeTime.GetHours(),
 																	  lifeTime.GetMinutes(),
																		  lifeTime.GetSeconds());
	}
 	m_strDomain = strArray[2];
 
 	UpdateData(FALSE);
}


/********************************************************************/
/*																	*/
/* Function name : OnDeleteitemData									*/
/* Description   : Delete item data when item is removed from list	*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::OnDeleteitemData(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	delete (LPTSTR)m_DataList.GetItemData(pNMListView->iItem);	

	if (pResult)
		*pResult = 0;
}


/********************************************************************/
/*																	*/
/* Function name : CreateBars										*/	
/* Description   : This function creates a toolbar and statusbar.	*/
/*																	*/
/********************************************************************/
BOOL CCookieViewDlg::CreateBars()
{
	// create the toolbar
	if(!m_wndToolBar.Create(this) || !m_wndToolBar.LoadToolBar(IDR_TOOLBAR2))
	{
		TRACE0("Failed to create toolbar\n");
		return FALSE;	
	}

	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() | CCS_NODIVIDER | CBRS_TOOLTIPS | CBRS_FLYBY);
	
	// make it flat.
	m_wndToolBar.ModifyStyle(0, m_wndToolBar.GetStyle()|TBSTYLE_FLAT);

	// create status bar at the bottom of the window
/*	if (m_wndStatusBar.Create(this))
	{
		m_wndStatusBar.SetIndicators(auIDStatusBar, sizeof(auIDStatusBar)/sizeof(UINT));
		
		m_wndStatusBar.SetPaneText(1, "", TRUE);
		m_wndStatusBar.SetPaneInfo(0, m_wndStatusBar.GetItemID(0), SBPS_STRETCH, NULL );
	}
*/
	// Set up hot bar image lists.
    CImageList	imageList;
	CBitmap		bitmap;

    // Create and set the normal toolbar image list.
	bitmap.LoadBitmap(IDB_TOOLBAR_COLD);
	imageList.Create(16, 16, ILC_COLORDDB|ILC_MASK, 4, 1);
	imageList.Add(&bitmap, RGB(255,0,255));
	m_wndToolBar.SendMessage(TB_SETIMAGELIST, 0, (LPARAM)imageList.m_hImageList);
	imageList.Detach();
	bitmap.Detach();

    // Create and set the normal toolbar image list.
	bitmap.LoadBitmap(IDB_TOOLBAR_HOT);
	imageList.Create(16, 16, ILC_COLORDDB|ILC_MASK, 4, 1);
	imageList.Add(&bitmap, RGB(255,0,255));
	m_wndToolBar.SendMessage(TB_SETHOTIMAGELIST, 0, (LPARAM)imageList.m_hImageList);
	imageList.Detach();
	bitmap.Detach();

	// Set the text for each button
	CToolBarCtrl& bar = m_wndToolBar.GetToolBarCtrl();

	int	nIndex = 0;
	TBBUTTON tb;

    for (nIndex = m_wndToolBar.GetToolBarCtrl().GetButtonCount() - 1; nIndex >= 0; nIndex--)
	{
		ZeroMemory(&tb, sizeof(TBBUTTON));
		m_wndToolBar.GetToolBarCtrl().GetButton(nIndex, &tb);

		// Do we have a separator?
		if ((tb.fsStyle & TBSTYLE_SEP) ==  TBSTYLE_SEP)
			continue;

		// Have we got a valid command id?
		if (tb.idCommand == 0)
			continue;

		// Get the resource string if there is one.
		CString strText;
		LPCTSTR lpszButtonText = NULL;
		CString	strButtonText("");
		_TCHAR	seps[] = "\n";

		strText.LoadString(tb.idCommand);

		if (!strText.IsEmpty())
		{
			lpszButtonText = _tcstok((LPTSTR)(LPCTSTR)strText, seps);

			while(lpszButtonText)
			{
				strButtonText = lpszButtonText;
				lpszButtonText = _tcstok(NULL, seps);
			}
		}

		if (!strButtonText.IsEmpty())
			m_wndToolBar.SetButtonText(nIndex, strButtonText);
	}

	// Resize the buttons so that the text will fit.
	CRect rc(0, 0, 0, 0);
	CSize sizeMax(0, 0);

	for (nIndex = m_wndToolBar.GetToolBarCtrl().GetButtonCount() - 1; nIndex >= 0; nIndex--)
	{
		m_wndToolBar.GetToolBarCtrl().GetItemRect(nIndex, rc);

		rc.NormalizeRect();
		sizeMax.cx = __max(rc.Size().cx, sizeMax.cx);
		sizeMax.cy = __max(rc.Size().cy, sizeMax.cy);
	}
	m_wndToolBar.SetSizes(sizeMax, CSize(16,16));

    // resize the dialog to make room for control bars
	CRect rcClientStart;
	CRect rcClientNow;
	GetClientRect(rcClientStart);
	RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, 
				   0, reposQuery, rcClientNow);

    // move all the controls so they are in the same relative position within the remaining client area 
	// as they would be with no control bars.
	CPoint ptOffset(rcClientNow.left - rcClientStart.left,
					rcClientNow.top - rcClientStart.top); 

	CRect  rcChild;					
	CWnd* pwndChild = GetWindow(GW_CHILD);
	while (pwndChild)
	{                               
		pwndChild->GetWindowRect(rcChild);
		ScreenToClient(rcChild);
		rcChild.OffsetRect(ptOffset);
		pwndChild->MoveWindow(rcChild, FALSE);
		pwndChild = pwndChild->GetNextWindow();
	}

	// adjust the dialog window dimensions
	CRect rcWindow;
	GetWindowRect(rcWindow);
	rcWindow.right += rcClientStart.Width() - rcClientNow.Width();
	rcWindow.bottom += rcClientStart.Height() - rcClientNow.Height();
	MoveWindow(rcWindow, FALSE);
	
	// position the control bars
	RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, 0);
    return TRUE;
}


/********************************************************************/
/*																	*/
/* Function name : OnEditDelete										*/	
/* Description   : Delete all selected cookies.						*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::OnEditDelete() 
{
	CCookieViewApp *pApp = (CCookieViewApp *)AfxGetApp();

	int nIndex = m_CookiesList.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);

	if ((nIndex != -1) && (pApp->m_bConfirmation))
	{
		if (MessageBox("Are you sure you want to delete all selected Cookies?", "Delete Cookies" , MB_YESNO | MB_ICONQUESTION) != IDYES)
		{
			return;
		}
	}

	// delete items
	while (nIndex != -1)
	{
		CString strFilename;
		
		strFilename.Format("%s\\%s", pApp->m_strCookieFolder, m_CookiesList.GetItemText(nIndex, 0));
		
		if (!DeleteFile(strFilename))
			break;

		m_CookiesList.DeleteItem(nIndex);
		
		nIndex = m_CookiesList.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	}
	m_CookiesList.SetItemState(0, LVIS_SELECTED | LVIS_FOCUSED , LVIS_SELECTED | LVIS_FOCUSED); 
}


/********************************************************************/
/*																	*/
/* Function name : OnEditSelectAll									*/	
/* Description   : Select all cookies.								*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::OnEditSelectAll() 
{
	for (int i = 0; i < m_CookiesList.GetItemCount(); i++)
	{
		m_CookiesList.SetItemState(i, LVIS_SELECTED, LVIS_SELECTED);
	}
}


/********************************************************************/
/*																	*/
/* Function name : OnViewRefresh									*/	
/* Description   : Refresh cookies list.							*/
/*																	*/
/********************************************************************/
void CCookieViewDlg::OnViewRefresh() 
{
	ShowCookies();	
}


/********************************************************************/
/*																	*/
/* Function name : PreTranslateMessage								*/	
/* Description   : Handle keyboard shortcuts						*/
/*																	*/
/********************************************************************/
BOOL CCookieViewDlg::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)	
	{	
		case WM_KEYDOWN:		
			switch(pMsg->wParam)		
			{	
				case VK_F3:
					OnEditFind();
					return TRUE;
				case VK_F5:
					OnViewRefresh();
					return TRUE;
				case VK_DELETE:  
					OnEditDelete();
					return TRUE;		
				case 'A':  
				case 'a':  
					if (GetKeyState(VK_CONTROL) < 0)	
						OnEditSelectAll();
					return TRUE;		
			}
		default:
			break;
	}			
	return CDialogResize::PreTranslateMessage(pMsg);
}

void CCookieViewDlg::OnHelpAbout() 
{
	CAboutDlg dlg;
	dlg.DoModal();
}


void CCookieViewDlg::OnEditFind() 
{
	static CString strText = "";

	CFindDlg dlg;
	
	dlg.m_strText = strText;

	if (dlg.DoModal() == IDOK)
	{
		strText = dlg.m_strText;
		strText.MakeLower();

		int nIndex = m_CookiesList.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED) + 1;

		// deselect all
		for (int i = 0; i < m_CookiesList.GetItemCount(); i++)
		{
			if (m_CookiesList.GetItemState(i, LVNI_SELECTED))
				m_CookiesList.SetItemState(i, ~LVIS_SELECTED, LVIS_SELECTED);
		}

		for (int j=nIndex; j < m_CookiesList.GetItemCount(); j++)
		{
			if (m_CookiesList.GetItemText(j, 0).Find(strText) != -1)
			{
				m_CookiesList.SetItemState(j, LVIS_SELECTED | LVIS_FOCUSED , LVIS_SELECTED | LVIS_FOCUSED); 
				m_CookiesList.EnsureVisible(j, FALSE);
				break;
			}
		}
	}
}
